""" Panoptes - AWS
Responsible to import analysis functions from AWS.
"""

from panoptes.aws import analysis
from panoptes.aws import attached
from panoptes.aws import authentication
from panoptes.aws import exceptions
from panoptes.aws import output
from panoptes.aws import whitelist


if __name__ == "__main__":
    pass
