""" Panoptes - Main Module

Responsible only to import the cloud respective folders, which were created to
organize the project properly.
"""

from panoptes import aws
from panoptes import cli
from panoptes import generic
from panoptes import panoptesctl


if __name__ == "__main__":
    pass
